LOOKUP_OPERATORS = {
    "exact": "=",
    "gt": ">",
    "gte": ">=",
    "lt": "<",
    "lte": "<=",
    "in": "IN",
    "like": "LIKE",
    "ilike": "ILIKE",
    "contains": "CONTAINS",
    "icontains": "CONTAINS",
    "startswith": "STARTSWITH",
    "istartswith": "STARTSWITH",
    "endswith": "ENDSWITH",
    "iendswith": "ENDSWITH",
    "match": "MATCH",
    "regex": "REGEX",
    "iregex": "REGEX",
    "isnull": "IS",
}
